# coding: utf-8

"""SteelConnection

Convienience objects for making REST API calls
to Riverbed SteelConnect Manager.

Usage:
    sconnect = steelconnection.SConAPI(scm_name, username, password)

    Optional keyword api_version can be used to specify an API version number.
    Currently there is only one API version: '1.0'.

    Once you have instantiated an object as shown above,
    you can use the object to make calls to the REST API.

    For example, to get all nodes in the realm:
    nodes = sconnect.config.get('nodes')
    ... or in a specifc org:
    nodes = sconnect.config.get('/org/' + orgid + '/nodes')  
    
    Any call that does not result in a success (HTTP status code 200)
    will raise an exception, so calls should be wrapped in a try/except pair.
"""


from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import getpass
import json
import requests
import sys
import traceback

from .__version__ import __version__
from .lookup import _LookUp
from .input_tools import get_username, get_password



class SConAPI(object):
    """Make REST API calls to Riverbed SteelConnect Manager."""

    def __init__(
        self,
        controller,
        username=None,
        password=None,
        api_version='1.0',
        exit_on_error=False,
        raise_on_failure=True,
    ):
        """Initialize attributes."""
        if not controller.endswith('.cc'):
            raise ValueError("SteelConnect Manager's name must end with '.cc'")
        self.api_version = api_version
        self.controller = controller
        self.exit_on_error = exit_on_error
        self.raise_on_failure = raise_on_failure
        self.username = get_username() if username is None else username
        self.password = get_password() if password is None else password
        self.session = requests.Session()
        self.result = None
        self.response = None
        self.headers = {
            'Accept': 'application/json',
            'Content-type': 'application/json',
        }
        self.lookup = _LookUp(self)
        self.__version__ = __version__

    def __bool__(self):
        """Return the success of the last request."""
        return False if self.response is None else self.response.ok 

    def __repr__(self):
        """Return a string consisting of class name, controller, and api."""
        details = ', '.join([
            "controller: '{0}'".format(self.controller),
            "api version: '{0}'".format(self.api_version),
            "response: '{0}'".format(self.response),
        ])
        return '{0}({1})'.format(self.__class__.__name__, details)

    def get(self, resource, params=None):
        r"""Send a GET request to the SteelConnect.Config API.

        :param str resource: api resource to get.
        :param dict params: (optional) Dictionary of query parameters.
        :returns: Dictionary or List of Dictionaries based on request.
        :rtype: dict, or list
        """
        return self._request(
            request_method=self.session.get,
            api='config',
            resource=resource,
            params=params,
        )

    def getstatus(self, resource, params=None):
        r"""Send a GET request to the SteelConnect.Reporting API.

        :param str resource: api resource to get.
        :param dict params: (optional) Dictionary of query parameters.
        :returns: Dictionary or List of Dictionaries based on request.
        :rtype: dict, or list
        """
        return self._request(
            request_method=self.session.get,
            api='reporting',
            resource=resource,
            params=params,
        )

    def delete(self, resource, data=None, params=None):
        r"""Send a DELETE request to the SteelConnect.Config API.

        :param str resource: api resource to get.
        :param dict data: (optional) Dictionary of 'body' data to be sent.
        :param dict params: (optional) Dictionary of query parameters.
        :returns: Dictionary or List of Dictionaries based on request.
        :rtype: dict, or list
        """
        return self._request(
            request_method=self.session.delete,
            api='config',
            resource=resource,
            data=data, 
        )

    def post(self, resource, data=None):
        r"""Send a POST request to the SteelConnect.Config API.

        :param str resource: api resource to get.
        :param dict data: (optional) Dictionary of 'body' data to be sent.
        :returns: Dictionary or List of Dictionaries based on request.
        :rtype: dict, or list
        """
        return self._request(
            request_method=self.session.post,
            api='config',
            resource=resource,
            data=data, 
        )

    def put(self, resource, data=None, params=None):
        r"""Send a PUT request to the SteelConnect.Config API.

        :param str resource: api resource to get.
        :param dict data: (optional) Dictionary of 'body' data to be sent.
        :param dict params: (optional) Dictionary of query parameters.
        :returns: Dictionary or List of Dictionaries based on request.
        :rtype: dict, or list
        """
        return self._request(
            request_method=self.session.put,
            api='config',
            resource=resource,
            data=data, 
        )

    def url(self, api, resource):
        """Combine attributes and resource as a url string."""
        resource = resource[1:] if resource.startswith('/') else resource
        return 'https://{0}/api/scm.{1}/{2}/{3}'.format(
            self.controller, api, self.api_version, resource,
        )

    def savefile(self, filename):
        """Save binary return data to a file."""
        with open(filename, 'wb') as f:
            f.write(self.response.content)
    
    def _request(self, request_method, api, resource, data=None, params=None):
        """Send request to controller and handle response."""
        self.response = self._make_request(
            request_method=request_method,
            api=api,
            resource=resource,
            data=data, 
            params=params,
        )
        if not self.response.ok:
            # work-around for bug in get:'/node/{node_id}/image_status' response
            if self.response.text and 'Queued' in self.response.text:
                return self.response.json()
            error = _error_string(self.response)
            if self.exit_on_error:
                print(error, file=sys.stderr)
                sys.exit(1)
            elif self.raise_on_failure:
                self._tb = traceback.extract_stack()
                raise RuntimeError(error)
            return {'error': error}
        if self.response.headers['Content-Type'] == 'application/octet-stream':
            return {
                'status': (
                    "Binary data returned.  "
                    "Use '.savefile(filename)' method "
                    "or access using '.response.content'."
                )
            }
        if not self.response.json():
            self.result = {}
        elif 'items' in self.response.json():
            self.result = self.response.json()['items']
        else:
            self.result = self.response.json()
        return self.result

    def _make_request(self, request_method, api, resource, data=None, params=None):
        """Send HTTP request to SteelConnect manager."""
        if data and isinstance(data, dict):
            data = json.dumps(data)
        try:
            response = request_method(
                url=self.url(api, resource),
                auth=(self.username, self.password),
                headers=self.headers,
                params=params,
                data=data,
            )
        except Exception as e:
            if self.exit_on_error:
                error = 'Connection to SteelConnect Manager failed:'
                print(error, file=sys.stderr)
                print(e, file=sys.stderr)
                sys.exit(1)
            else:
                raise e
        return response


def _error_string(response):
    """Summarize error conditions and return as a string."""
    details = ''
    if response.text:
        try:
            details = response.json()
            details = details.get('error', {}).get('message', '')
        except ValueError:
            pass
    error = '\t{0} - {1}{2}\nURL:      \t{3}\nData Sent:\t{4}'.format(
        response.status_code,
        response.reason,
        '\nDetails:\t' + details if details else '',
        response.url,
        repr(response.request.body),
    )
    return error



__title__ = 'steelconnection'
__description__ = 'Simplify access to the Riverbed SteelConnect REST API.'
__url__ = 'https://github.com/grelleum/SteelConnection'
__version__ = '0.8.9'
__author__ = 'Greg Mueller'
__author_email__ = 'steelconnection@grelleum.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2018 Greg Mueller'



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import steelconnection.version

home = 'https://pypi.org/project/steelconnection/'

if __name__ == '__main__':
    print('SteelConnection version:', steelconnection.version.__version__)
    print('Documentation available at', home)



# coding: utf-8

"""SteelConnection

Convienience object for ID value lookups based on common criteria.

Should be instantiated within a steelconnection object.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import sys
import warnings


class _LookUp(object):
    """Provide convienience tools to lookup objects."""

    def __init__(self, sconnection):
        """Obtain access to SteelConect Manager."""
        self.sconnection = sconnection

    def _lookup(self, domain, value, key, return_value='id'):
        """
        Generic lookup function.
        Given a resource type (domain), a value to match, and a key to lookup,
        returns a tuple consisting of an object ID and the actual object.
        """
        items = self.sconnection.get(domain)
        for item in items:
            key_value = item.get(key, '')
            if key_value and value in key_value:
                self.sconnection.result = item.get(return_value, '')
                return self.sconnection.result, item
        return None, {'status': 'Failed', 'message': 'Not Found'}

    def node(self, serial, key='serial'):
        """
        Returns a tuple of node_id and node
        matching a provided appliance serial number.
        """
        return self._lookup(domain='nodes', value=serial, key=key)

    def org(self, name, key='name'):
        """
        Returns a tuple of org_id and org
        matching a provided organization short name.
        """
        return self._lookup(domain='orgs', value=name, key=key)

    def site(self, name, orgid=None, key='name'):
        """
        Returns a tuple of site_id and site
        matching a provided site short name and org_id.
        """
        if not orgid:
            raise ValueError('orgid required when looking up a site.')
        resource = '/'.join(('org', orgid, 'sites'))
        return self._lookup(domain=resource, value=name, key=key)

    # Methods below are deprecated.

    def nodeid(self, serial, key='serial'):
        """deprecated:
        Return node id that matches appliance serial number provided.
        """
        warnings.warn(
            'lookup.nodeid deprecated: use lookup.node instead',
            DeprecationWarning
        )
        result, _details = self._lookup(domain='nodes', value=serial, key=key)
        return result

    def orgid(self, name, key='name'):
        """deprecated:
        Return org id that matches organization short name provided.
        """
        warnings.warn(
            'lookup.orgid deprecated: use lookup.org instead',
            DeprecationWarning
        )
        result, _details = self._lookup(domain='orgs', value=name, key=key)
        return result

    def siteid(self, name, orgid=None, key='name'):
        """deprecated:
        Return site id that matches site short name
        based on the organization provided.
        """
        warnings.warn(
            'lookup.siteid deprecated: use lookup.site instead',
            DeprecationWarning
        )
        if not orgid:
            raise ValueError('orgid required when looking up a site.')
        resource = '/'.join(('org', orgid, 'sites'))
        result, _details = self._lookup(domain=resource, value=name, key=key)
        return result



import re
import setuptools

from steelconnection.__version__ import __author__, __author_email__
from steelconnection.__version__ import __copyright__, __description__
from steelconnection.__version__ import __license__, __title__
from steelconnection.__version__ import __url__, __version__


keywords = ['SteelConnect', 'REST', 'API', 'Riverbed', 'Grelleum']
base_url = 'https://github.com/grelleum/SteelConnection/archive/'

classifiers=(
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent'
)

with open('README.md', 'rt') as f:
    long_description = f.read()

download_url = base_url + __version__ + '.tar.gz'

setuptools.setup(
    name=__title__,
    version=__version__,
    author=__author__,
    author_email=__author_email__,
    description=__description__,
    long_description=long_description,
    long_description_content_type='text/markdown',
    url=__url__,
    download_url=download_url,
    keywords = keywords,
    packages=setuptools.find_packages(),
    classifiers=classifiers
)

# setuptools.setup(
#     name=name,
#     version=version,
#     author=author,
#     author_email=author_email,
#     description=description,
#     long_description=long_description,
#     long_description_content_type='text/markdown',
#     url=url,
#     download_url=download_url,
#     keywords = keywords,
#     packages=setuptools.find_packages(),
#     classifiers=classifiers
# )


# -*- coding: utf-8 -*-

"""
SteelConnection:
Simplify access to the Riverbed REST API.

usage:

import steelconnection
sconnect = steelconnection.SConAPI('REALM.riverbed.cc')
org_id, org = sconnect.lookup.org('MyOrgName')
nodes = sconnect.get('org/' + org_id + '/nodes')

Full documentation available at https://pypi.org/project/steelconnection/

:copyright: (c) 2018 by Greg Mueller.
:license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from .steelconnection import SConAPI
from .input_tools import get_input
from .input_tools import get_username
from .input_tools import get_password

import logging
logging.getLogger(__name__).addHandler(logging.NullHandler())

from .__version__ import __author__, __author_email__
from .__version__ import __copyright__, __description__
from .__version__ import __license__, __title__
from .__version__ import __url__, __version__


__all__ = (
    'SConAPI',
    'get_input',
    'get_username',
    'get_password',
)
